// SerialPortsTest.js
//  This module handles serial port testing

//#region Dependencies, Constants and Globals
//#region Dependencies
var fs = require('fs');
var cp = require('child_process');
var moment = require('moment');
var path = require('path');
var serialport = require("serialport");
//#endregion Dependencies

//#region Constants
var SERIAL_PORTS_TOTAL = 8;

var port0 = new serialport("/dev/ttyUSB0", { baudRate: 9600, dataBits: 8, stopBits: 1, parity: 'none', parser: serialport.parsers.raw, autoOpen: false });
var port1 = new serialport("/dev/ttyUSB1", { baudRate: 9600, dataBits: 8, stopBits: 1, parity: 'none', parser: serialport.parsers.raw, autoOpen: false });
var port2 = new serialport("/dev/ttyUSB2", { baudRate: 9600, dataBits: 8, stopBits: 1, parity: 'none', parser: serialport.parsers.raw, autoOpen: false });
var port3 = new serialport("/dev/ttyUSB3", { baudRate: 9600, dataBits: 8, stopBits: 1, parity: 'none', parser: serialport.parsers.raw, autoOpen: false });
var port4 = new serialport("/dev/ttyUSB4", { baudRate: 9600, dataBits: 8, stopBits: 1, parity: 'none', parser: serialport.parsers.raw, autoOpen: false });
var port5 = new serialport("/dev/ttyUSB5", { baudRate: 9600, dataBits: 8, stopBits: 1, parity: 'none', parser: serialport.parsers.raw, autoOpen: false });
var port6 = new serialport("/dev/ttyUSB6", { baudRate: 9600, dataBits: 8, stopBits: 1, parity: 'none', parser: serialport.parsers.raw, autoOpen: false });
var port7 = new serialport("/dev/ttyUSB7", { baudRate: 9600, dataBits: 8, stopBits: 1, parity: 'none', parser: serialport.parsers.raw, autoOpen: false });

var SerialPorts = [
    { "address": 0, "port": port0, "status": "Unknown", "received": "" },
    { "address": 1, "port": port1, "status": "Unknown", "received": "" },
    { "address": 2, "port": port2, "status": "Unknown", "received": "" },
    { "address": 3, "port": port3, "status": "Unknown", "received": "" },
    { "address": 4, "port": port4, "status": "Unknown", "received": "" },
    { "address": 5, "port": port5, "status": "Unknown", "received": "" },
    { "address": 6, "port": port6, "status": "Unknown", "received": "" },
    { "address": 7, "port": port7, "status": "Unknown", "received": "" }
];

//#endregion Constants

//#region Globals
var LogManager;// Used for logging
var SocketIoManager;// Used for messaging
//#endregion Globals
//#endregion Dependencies, Constants and Globals

// Constructor
//#region Serial Port Test Initialization

// Initialize the analog outputs upon system start
//      - Checks if database table exists, if not creates the table and fills it with default data for each point
//      - Else reads from the database and updates analogOutputsArray
exports.InitializeSerialPortsTest = function (io, logger, callback) {

    // Initialize constants
    //#region Initialize constants
    SocketIoManager = io;
    LogManager = logger;
    //#endregion Initialize constants
};
//#endregion Serial Port Test Initialization

//#region Serial Port Test API

// GetSerialPorts
exports.GetSerialPorts = function (callback) {
    LogManager.info("SerialPortsTest.GetSerialPort: returning Serial Ports");
    callback(SerialPorts);
};

// OpenSerialPort
exports.OpenSerialPort = function (_port, callback) {
    SerialPorts[_port].port.open(function (error) {
        if (error) {
            SerialPorts[_port].status = "Error";
            SocketIoManager.emit("SerialPortTest", SerialPorts[_port]);
            LogManager.error("SerialPortsTest.OpenSerialPort: Port " + _port + " open error=" + error);
            callback({ "result": "error" });
        } else {
            SerialPorts[_port].status = "Open";
            SocketIoManager.emit("SerialPortTest", SerialPorts[_port]);
            LogManager.info("SerialPortsTest.OpenSerialPort: Port " + _port + " opened!");
            callback({ "result": "ok" });
        }
    });
};

// WriteSerialPort
exports.WriteSerialPort = function (_port, callback) {

    if (SerialPorts[_port].port.isOpen()) {
        var data = "Sent from port" + _port;
        SerialPorts[_port].port.write(data,function (error) {
            if (error) {
                LogManager.error("SerialPortsTest.WriteSerialPort: Port " + _port + " write error=" + error);
                callback({ "result": "error" });
            } else {
                LogManager.info("SerialPortsTest.WriteSerialPort: Port " + _port + " wrote data='" + data + "'");
                callback({ "result": "ok" });
            }
        });
    }
};

// CloseSerialPort
exports.CloseSerialPort = function (_port, callback) {

    if (SerialPorts[_port].port.isOpen()) {
        SerialPorts[_port].port.close(function (error) {
            if (error) {
                SerialPorts[_port].status = "Error";
                SocketIoManager.emit("SerialPortTest", SerialPorts[_port]);
                LogManager.error("SerialPortsTest.CloseSerialPort: Port " + _port + " close error=" + error);
                callback({ "result": "error" });
            } else {
                SerialPorts[_port].status = "Closed";
                SocketIoManager.emit("SerialPortTest", SerialPorts[_port]);
                LogManager.info("SerialPortsTest.CloseSerialPort: Port " + _port + " closed!");
                callback({ "result": "ok" });
            }
        });
    }
};

//#region Handle incoming data
port0.on("data", function (data) {
    LogManager.fatal("SerialPortsTest.OnData: Port 0 data received='" + data + "'");
    SerialPorts[0].received = data.toString('utf8');
    SocketIoManager.emit("SerialPortTest", SerialPorts[0]);
    setTimeout(function () {
        SerialPorts[0].received = "";
        SocketIoManager.emit("SerialPortTest", SerialPorts[0]);
    }, 3000);
});
port1.on("data", function (data) {
    LogManager.fatal("SerialPortsTest.OnData: Port 1 data received='" + data + "'");
    SerialPorts[1].received = data.toString('utf8');
    SocketIoManager.emit("SerialPortTest", SerialPorts[1]);
    setTimeout(function () {
        SerialPorts[1].received = "";
        SocketIoManager.emit("SerialPortTest", SerialPorts[1]);
    }, 3000);
});
port2.on("data", function (data) {
    LogManager.fatal("SerialPortsTest.OnData: Port 2 data received='" + data + "'");
    SerialPorts[2].received = data.toString('utf8');
    SocketIoManager.emit("SerialPortTest", SerialPorts[2]);
    setTimeout(function () {
        SerialPorts[2].received = "";
        SocketIoManager.emit("SerialPortTest", SerialPorts[2]);
    }, 3000);
});
port3.on("data", function (data) {
    LogManager.fatal("SerialPortsTest.OnData: Port 3 data received='" + data + "'");
    SerialPorts[3].received = data.toString('utf8');
    SocketIoManager.emit("SerialPortTest", SerialPorts[3]);
    setTimeout(function () {
        SerialPorts[3].received = "";
        SocketIoManager.emit("SerialPortTest", SerialPorts[3]);
    }, 3000);
});
port4.on("data", function (data) {
    LogManager.fatal("SerialPortsTest.OnData: Port 4 data received='" + data + "'");
    SerialPorts[4].received = data.toString('utf8');
    SocketIoManager.emit("SerialPortTest", SerialPorts[4]);
    setTimeout(function () {
        SerialPorts[4].received = "";
        SocketIoManager.emit("SerialPortTest", SerialPorts[4]);
    }, 3000);
});
port5.on("data", function (data) {
    LogManager.fatal("SerialPortsTest.OnData: Port 5 data received='" + data + "'");
    SerialPorts[5].received = data.toString('utf8');
    SocketIoManager.emit("SerialPortTest", SerialPorts[5]);
    setTimeout(function () {
        SerialPorts[5].received = "";
        SocketIoManager.emit("SerialPortTest", SerialPorts[5]);
    }, 3000);
});
port6.on("data", function (data) {
    LogManager.fatal("SerialPortsTest.OnData: Port 6 data received='" + data + "'");
    SerialPorts[6].received = data.toString('utf8');
    SocketIoManager.emit("SerialPortTest", SerialPorts[6]);
    setTimeout(function () {
        SerialPorts[6].received = "";
        SocketIoManager.emit("SerialPortTest", SerialPorts[6]);
    }, 3000);
});
port7.on("data", function (data) {
    LogManager.fatal("SerialPortsTest.OnData: Port 7 data received='" + data + "'");
    SerialPorts[7].received = data.toString('utf8');
    SocketIoManager.emit("SerialPortTest", SerialPorts[7]);
    setTimeout(function () {
        SerialPorts[7].received = "";
        SocketIoManager.emit("SerialPortTest", SerialPorts[7]);
    }, 3000);
});
//#endregion Handle incoming data

//#endregion Serial Port API